@@ -6,7 +6,7 @@
 # Derived from IPython.utils.path, which is
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
-
+from __future__ import annotations
 
 import errno
 import os
@@ -17,7 +17,7 @@ import tempfile
 import warnings
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional
+from typing import Any, Iterator, Literal, Optional, overload
 
 import platformdirs
 
@@ -25,14 +25,27 @@ from .utils import deprecation
 
 pjoin = os.path.join
 
-# Capitalize Jupyter in paths only on Windows and MacOS
-APPNAME = "Jupyter" if sys.platform in ("win32", "darwin") else "jupyter"
+# Capitalize Jupyter in paths only on Windows and MacOS (when not in Homebrew)
+if sys.platform == "win32" or (
+    sys.platform == "darwin" and not sys.prefix.startswith("/opt/homebrew")
+):
+    APPNAME = "Jupyter"
+else:
+    APPNAME = "jupyter"
 
 # UF_HIDDEN is a stat flag not defined in the stat module.
 # It is used by BSD to indicate hidden files.
 UF_HIDDEN = getattr(stat, "UF_HIDDEN", 32768)
 
 
+@overload
+def envset(name: str, default: bool = False) -> bool: ...
+
+
+@overload
+def envset(name: str, default: Literal[None]) -> Optional[bool]: ...
+
+
 def envset(name: str, default: Optional[bool] = False) -> Optional[bool]:
     """Return the boolean value of a given environment variable.
 
@@ -53,19 +66,18 @@ def use_platform_dirs() -> bool:
     We plan for this to default to False in jupyter_core version 5 and to True
     in jupyter_core version 6.
     """
-    return envset("JUPYTER_PLATFORM_DIRS", False)  # type:ignore[return-value]
+    return envset("JUPYTER_PLATFORM_DIRS", False)
 
 
 def get_home_dir() -> str:
     """Get the real path of the home directory"""
-    homedir = os.path.expanduser("~")
+    homedir = Path("~").expanduser()
     # Next line will make things work even when /home/ is a symlink to
     # /usr/home as it is on FreeBSD, for example
-    homedir = str(Path(homedir).resolve())
-    return homedir
+    return str(Path(homedir).resolve())
 
 
-_dtemps: Dict[str, str] = {}
+_dtemps: dict[str, str] = {}
 
 
 def _do_i_own(path: str) -> bool:
@@ -80,10 +92,10 @@ def _do_i_own(path: str) -> bool:
     # not always implemented or available
     try:
         return p.owner() == os.getlogin()
-    except Exception:  # noqa
+    except Exception:  # noqa: S110
         pass
 
-    if hasattr(os, 'geteuid'):
+    if hasattr(os, "geteuid"):
         try:
             st = p.stat()
             return st.st_uid == os.geteuid()
@@ -99,7 +111,7 @@ def prefer_environment_over_user() -> bool:
     """Determine if environment-level paths should take precedence over user-level paths."""
     # If JUPYTER_PREFER_ENV_PATH is defined, that signals user intent, so return its value
     if "JUPYTER_PREFER_ENV_PATH" in os.environ:
-        return envset("JUPYTER_PREFER_ENV_PATH")  # type:ignore[return-value]
+        return envset("JUPYTER_PREFER_ENV_PATH")
 
     # If we are in a Python virtualenv, default to True (see https://docs.python.org/3/library/venv.html#venv-def)
     if sys.prefix != sys.base_prefix and _do_i_own(sys.prefix):
@@ -169,19 +181,17 @@ def jupyter_data_dir() -> str:
     home = get_home_dir()
 
     if sys.platform == "darwin":
-        return os.path.join(home, "Library", "Jupyter")
-    elif os.name == "nt":
+        return str(Path(home, "Library", "Jupyter"))
+    if sys.platform == "win32":
         appdata = os.environ.get("APPDATA", None)
         if appdata:
             return str(Path(appdata, "jupyter").resolve())
-        else:
-            return pjoin(jupyter_config_dir(), "data")
-    else:
-        # Linux, non-OS X Unix, AIX, etc.
-        xdg = env.get("XDG_DATA_HOME", None)
-        if not xdg:
-            xdg = pjoin(home, ".local", "share")
-        return pjoin(xdg, "jupyter")
+        return pjoin(jupyter_config_dir(), "data")
+    # Linux, non-OS X Unix, AIX, etc.
+    xdg = env.get("XDG_DATA_HOME", None)
+    if not xdg:
+        xdg = pjoin(home, ".local", "share")
+    return pjoin(xdg, "jupyter")
 
 
 def jupyter_runtime_dir() -> str:
@@ -200,10 +210,22 @@ def jupyter_runtime_dir() -> str:
     return pjoin(jupyter_data_dir(), "runtime")
 
 
+# %PROGRAMDATA% is not safe by default, require opt-in to trust it
+_use_programdata: bool = envset("JUPYTER_USE_PROGRAMDATA")
+# _win_programdata is a path str if we're using it, None otherwise
+_win_programdata: str | None = None
+if os.name == "nt" and _use_programdata:
+    _win_programdata = os.environ.get("PROGRAMDATA", None)
+
+
 if use_platform_dirs():
-    SYSTEM_JUPYTER_PATH = platformdirs.site_data_dir(
-        APPNAME, appauthor=False, multipath=True
-    ).split(os.pathsep)
+    if os.name == "nt" and not _use_programdata:
+        # default PROGRAMDATA used by site_* is not safe by default on Windows
+        SYSTEM_JUPYTER_PATH = [str(Path(sys.prefix, "share", "jupyter"))]
+    else:
+        SYSTEM_JUPYTER_PATH = platformdirs.site_data_dir(
+            APPNAME, appauthor=False, multipath=True
+        ).split(os.pathsep)
 else:
     deprecation(
         "Jupyter is migrating its paths to use standard platformdirs\n"
@@ -213,34 +235,55 @@ else:
         "The use of platformdirs will be the default in `jupyter_core` v6"
     )
     if os.name == "nt":
-        programdata = os.environ.get("PROGRAMDATA", None)
-        if programdata:
-            SYSTEM_JUPYTER_PATH = [pjoin(programdata, "jupyter")]
-        else:  # PROGRAMDATA is not defined by default on XP.
-            SYSTEM_JUPYTER_PATH = [os.path.join(sys.prefix, "share", "jupyter")]
+        # PROGRAMDATA is not defined by default on XP, and not safe by default
+        if _win_programdata:
+            SYSTEM_JUPYTER_PATH = [pjoin(_win_programdata, "jupyter")]
+        else:
+            SYSTEM_JUPYTER_PATH = [str(Path(sys.prefix, "share", "jupyter"))]
     else:
         SYSTEM_JUPYTER_PATH = [
             "/usr/local/share/jupyter",
             "/usr/share/jupyter",
         ]
 
-ENV_JUPYTER_PATH: List[str] = [os.path.join(sys.prefix, "share", "jupyter")]
+ENV_JUPYTER_PATH: list[str] = [str(Path(sys.prefix, "share", "jupyter"))]
 
 
-def jupyter_path(*subdirs: str) -> List[str]:
-    """Return a list of directories to search for data files
+def jupyter_path(*subdirs: str) -> list[str]:
+    """Return a list of directories to search for data files.
 
-    JUPYTER_PATH environment variable has highest priority.
+    There are four sources of paths to search:
+
+    - $JUPYTER_PATH environment variable (always highest priority)
+    - user directories (e.g. ~/.local/share/jupyter)
+    - environment directories (e.g. {sys.prefix}/share/jupyter)
+    - system-wide paths (e.g. /usr/local/share/jupyter)
+
+    JUPYTER_PATH environment variable has highest priority, if defined,
+    and is purely additive.
 
     If the JUPYTER_PREFER_ENV_PATH environment variable is set, the environment-level
     directories will have priority over user-level directories.
+    You can also set JUPYTER_PREFER_ENV_PATH=0 to explicitly prefer user directories.
+    If Jupyter detects that you are in a virtualenv or conda environment,
+    environment paths are also preferred to user paths,
+    otherwise user paths are preferred to environment paths.
 
     If the Python site.ENABLE_USER_SITE variable is True, we also add the
     appropriate Python user site subdirectory to the user-level directories.
 
+    Finally, system-wide directories, such as `/usr/local/share/jupyter` are searched.
 
     If ``*subdirs`` are given, that subdirectory will be added to each element.
 
+
+    .. versionchanged:: 5.8
+
+        On Windows, %PROGRAMDATA% will be used as a system-wide path only if
+        the JUPYTER_USE_PROGRAMDATA env is set.
+        By default, there is no default system-wide path on Windows and the env path
+        is used instead.
+
     Examples:
 
     >>> jupyter_path()
@@ -249,7 +292,7 @@ def jupyter_path(*subdirs: str) -> List[str]:
     ['~/.local/jupyter/kernels', '/usr/local/share/jupyter/kernels']
     """
 
-    paths: List[str] = []
+    paths: list[str] = []
 
     # highest priority is explicit environment variable
     if os.environ.get("JUPYTER_PATH"):
@@ -264,11 +307,17 @@ def jupyter_path(*subdirs: str) -> List[str]:
         userbase = site.getuserbase() if hasattr(site, "getuserbase") else site.USER_BASE
 
         if userbase:
-            userdir = os.path.join(userbase, "share", "jupyter")
+            userdir = str(Path(userbase, "share", "jupyter"))
             if userdir not in user:
                 user.append(userdir)
 
-    env = [p for p in ENV_JUPYTER_PATH if p not in SYSTEM_JUPYTER_PATH]
+    # Windows usually doesn't have a 'system' prefix,
+    # so 'system' and 'env' are the same
+    # make sure that env can still be preferred in this case
+    if ENV_JUPYTER_PATH == SYSTEM_JUPYTER_PATH:
+        env = ENV_JUPYTER_PATH
+    else:
+        env = [p for p in ENV_JUPYTER_PATH if p not in SYSTEM_JUPYTER_PATH]
 
     if prefer_environment_over_user():
         paths.extend(env)
@@ -277,8 +326,10 @@ def jupyter_path(*subdirs: str) -> List[str]:
         paths.extend(user)
         paths.extend(env)
 
-    # finally, system
-    paths.extend(SYSTEM_JUPYTER_PATH)
+    # finally, add system paths (can overlap with env, so avoid duplicates)
+    for _path in SYSTEM_JUPYTER_PATH:
+        if _path not in paths:
+            paths.append(_path)
 
     # add subdir, if requested
     if subdirs:
@@ -287,25 +338,28 @@ def jupyter_path(*subdirs: str) -> List[str]:
 
 
 if use_platform_dirs():
-    SYSTEM_CONFIG_PATH = platformdirs.site_config_dir(
-        APPNAME, appauthor=False, multipath=True
-    ).split(os.pathsep)
-else:
-    if os.name == "nt":  # noqa
-        programdata = os.environ.get("PROGRAMDATA", None)
-        if programdata:  # noqa
-            SYSTEM_CONFIG_PATH = [os.path.join(programdata, "jupyter")]
-        else:  # PROGRAMDATA is not defined by default on XP.
-            SYSTEM_CONFIG_PATH = []
+    if os.name == "nt" and not _use_programdata:
+        # default PROGRAMDATA is not safe by default on Windows
+        SYSTEM_CONFIG_PATH = []
     else:
-        SYSTEM_CONFIG_PATH = [
-            "/usr/local/etc/jupyter",
-            "/etc/jupyter",
-        ]
-ENV_CONFIG_PATH: List[str] = [os.path.join(sys.prefix, "etc", "jupyter")]
+        SYSTEM_CONFIG_PATH = platformdirs.site_config_dir(
+            APPNAME, appauthor=False, multipath=True
+        ).split(os.pathsep)
+elif os.name == "nt":
+    # PROGRAMDATA is not defined by default on XP, and not safe by default
+    if _win_programdata:
+        SYSTEM_CONFIG_PATH = [str(Path(_win_programdata, "jupyter"))]
+    else:
+        SYSTEM_CONFIG_PATH = []
+else:
+    SYSTEM_CONFIG_PATH = [
+        "/usr/local/etc/jupyter",
+        "/etc/jupyter",
+    ]
+ENV_CONFIG_PATH: list[str] = [str(Path(sys.prefix, "etc", "jupyter"))]
 
 
-def jupyter_config_path() -> List[str]:
+def jupyter_config_path() -> list[str]:
     """Return the search path for Jupyter config files as a list.
 
     If the JUPYTER_PREFER_ENV_PATH environment variable is set, the
@@ -314,12 +368,27 @@ def jupyter_config_path() -> List[str]:
 
     If the Python site.ENABLE_USER_SITE variable is True, we also add the
     appropriate Python user site subdirectory to the user-level directories.
+
+    Finally, system-wide directories such as `/usr/local/etc/jupyter` are searched.
+
+
+    .. versionchanged:: 5.8
+
+        On Windows, %PROGRAMDATA% will be used as a system-wide path only if
+        the JUPYTER_USE_PROGRAMDATA env is set.
+        By default, there is no system-wide config path on Windows.
+
+    Examples:
+
+    >>> jupyter_config_path()
+    ['~/.jupyter', '~/.local/etc/jupyter', '/usr/local/etc/jupyter', '/etc/jupyter']
+
     """
     if os.environ.get("JUPYTER_NO_CONFIG"):
         # jupyter_config_dir makes a blank config when JUPYTER_NO_CONFIG is set.
         return [jupyter_config_dir()]
 
-    paths: List[str] = []
+    paths: list[str] = []
 
     # highest priority is explicit environment variable
     if os.environ.get("JUPYTER_CONFIG_PATH"):
@@ -334,7 +403,7 @@ def jupyter_config_path() -> List[str]:
         userbase = site.getuserbase() if hasattr(site, "getuserbase") else site.USER_BASE
 
         if userbase:
-            userdir = os.path.join(userbase, "etc", "jupyter")
+            userdir = str(Path(userbase, "etc", "jupyter"))
             if userdir not in user:
                 user.append(userdir)
 
@@ -379,28 +448,31 @@ def is_file_hidden_win(abs_path: str, stat_res: Optional[Any] = None) -> bool:
         The result of calling stat() on abs_path. If not passed, this function
         will call stat() internally.
     """
-    if os.path.basename(abs_path).startswith("."):
+    if Path(abs_path).name.startswith("."):
         return True
 
     if stat_res is None:
         try:
-            stat_res = os.stat(abs_path)
+            stat_res = Path(abs_path).stat()
         except OSError as e:
             if e.errno == errno.ENOENT:
                 return False
             raise
 
     try:
-        if stat_res.st_file_attributes & stat.FILE_ATTRIBUTE_HIDDEN:  # type:ignore
+        if (
+            stat_res.st_file_attributes  # type:ignore[union-attr]
+            & stat.FILE_ATTRIBUTE_HIDDEN  # type:ignore[attr-defined]
+        ):
             return True
     except AttributeError:
         # allow AttributeError on PyPy for Windows
         # 'stat_result' object has no attribute 'st_file_attributes'
         # https://foss.heptapod.net/pypy/pypy/-/issues/3469
         warnings.warn(
-            "hidden files are not detectable on this system, so no file will be marked as hidden."
+            "hidden files are not detectable on this system, so no file will be marked as hidden.",
+            stacklevel=2,
         )
-        pass
 
     return False
 
@@ -421,19 +493,19 @@ def is_file_hidden_posix(abs_path: str, stat_res: Optional[Any] = None) -> bool:
         The result of calling stat() on abs_path. If not passed, this function
         will call stat() internally.
     """
-    if os.path.basename(abs_path).startswith("."):
+    if Path(abs_path).name.startswith("."):
         return True
 
     if stat_res is None or stat.S_ISLNK(stat_res.st_mode):
         try:
-            stat_res = os.stat(abs_path)
+            stat_res = Path(abs_path).stat()
         except OSError as e:
             if e.errno == errno.ENOENT:
                 return False
             raise
 
     # check that dirs can be listed
-    if stat.S_ISDIR(stat_res.st_mode):  # type:ignore[misc]  # noqa
+    if stat.S_ISDIR(stat_res.st_mode):  # noqa: SIM102
         # use x-access, not actual listing, in case of slow/large listings
         if not os.access(abs_path, os.X_OK | os.R_OK):
             return True
@@ -483,15 +555,15 @@ def is_hidden(abs_path: str, abs_root: str = "") -> bool:
     if not abs_root:
         abs_root = abs_path.split(os.sep, 1)[0] + os.sep
     inside_root = abs_path[len(abs_root) :]
-    if any(part.startswith(".") for part in inside_root.split(os.sep)):
+    if any(part.startswith(".") for part in Path(inside_root).parts):
         return True
 
     # check UF_HIDDEN on any location up to root.
     # is_file_hidden() already checked the file, so start from its parent dir
-    path = os.path.dirname(abs_path)
+    path = str(Path(abs_path).parent)
     while path and path.startswith(abs_root) and path != abs_root:
-        if not exists(path):
-            path = os.path.dirname(path)
+        if not Path(path).exists():
+            path = str(Path(path).parent)
             continue
         try:
             # may fail on Windows junctions
@@ -500,7 +572,7 @@ def is_hidden(abs_path: str, abs_root: str = "") -> bool:
             return True
         if getattr(st, "st_flags", 0) & UF_HIDDEN:
             return True
-        path = os.path.dirname(path)
+        path = str(Path(path).parent)
 
     return False
 
@@ -546,9 +618,10 @@ def win32_restrict_file_to_user(fname: str) -> None:
 
     sd.SetSecurityDescriptorDacl(1, dacl, 0)
     win32security.SetFileSecurity(fname, win32security.DACL_SECURITY_INFORMATION, sd)
+    return None
 
 
-def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
+def _win32_restrict_file_to_user_ctypes(fname: str) -> None:
     """Secure a windows file to read-only access for the user.
 
     Follows guidance from win32 library creator:
@@ -614,7 +687,7 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
     PACL = ctypes.POINTER(ACL)
     PSECURITY_DESCRIPTOR = ctypes.POINTER(wintypes.BYTE)
 
-    def _nonzero_success(result, func, args):
+    def _nonzero_success(result: int, func: Any, args: Any) -> Any:  # noqa: ARG001
         if not result:
             raise ctypes.WinError(ctypes.get_last_error())  # type:ignore[attr-defined]
         return args
@@ -716,7 +789,7 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
         wintypes.DWORD,  # DWORD dwAclRevision
     )
 
-    def CreateWellKnownSid(WellKnownSidType):
+    def CreateWellKnownSid(WellKnownSidType: Any) -> Any:
         # return a SID for predefined aliases
         pSid = (ctypes.c_char * 1)()
         cbSid = wintypes.DWORD()
@@ -729,7 +802,7 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
             advapi32.CreateWellKnownSid(WellKnownSidType, None, pSid, ctypes.byref(cbSid))
         return pSid[:]
 
-    def GetUserNameEx(NameFormat):
+    def GetUserNameEx(NameFormat: Any) -> Any:
         # return the user or other security principal associated with
         # the calling thread
         nSize = ctypes.pointer(ctypes.c_ulong(0))
@@ -744,7 +817,7 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
         secur32.GetUserNameExW(NameFormat, lpNameBuffer, nSize)
         return lpNameBuffer.value
 
-    def LookupAccountName(lpSystemName, lpAccountName):
+    def LookupAccountName(lpSystemName: Any, lpAccountName: Any) -> Any:
         # return a security identifier (SID) for an account on a system
         # and the name of the domain on which the account was found
         cbSid = wintypes.DWORD(0)
@@ -779,12 +852,12 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
             raise ctypes.WinError()  # type:ignore[attr-defined]
         return pSid, lpReferencedDomainName.value, peUse.value
 
-    def AddAccessAllowedAce(pAcl, dwAceRevision, AccessMask, pSid):
+    def AddAccessAllowedAce(pAcl: Any, dwAceRevision: Any, AccessMask: Any, pSid: Any) -> Any:
         # add an access-allowed access control entry (ACE)
         # to an access control list (ACL)
         advapi32.AddAccessAllowedAce(pAcl, dwAceRevision, AccessMask, pSid)
 
-    def GetFileSecurity(lpFileName, RequestedInformation):
+    def GetFileSecurity(lpFileName: Any, RequestedInformation: Any) -> Any:
         # return information about the security of a file or directory
         nLength = wintypes.DWORD(0)
         try:
@@ -810,15 +883,19 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
         )
         return pSecurityDescriptor
 
-    def SetFileSecurity(lpFileName, RequestedInformation, pSecurityDescriptor):
+    def SetFileSecurity(
+        lpFileName: Any, RequestedInformation: Any, pSecurityDescriptor: Any
+    ) -> Any:
         # set the security of a file or directory object
         advapi32.SetFileSecurityW(lpFileName, RequestedInformation, pSecurityDescriptor)
 
-    def SetSecurityDescriptorDacl(pSecurityDescriptor, bDaclPresent, pDacl, bDaclDefaulted):
+    def SetSecurityDescriptorDacl(
+        pSecurityDescriptor: Any, bDaclPresent: Any, pDacl: Any, bDaclDefaulted: Any
+    ) -> Any:
         # set information in a discretionary access control list (DACL)
         advapi32.SetSecurityDescriptorDacl(pSecurityDescriptor, bDaclPresent, pDacl, bDaclDefaulted)
 
-    def MakeAbsoluteSD(pSelfRelativeSecurityDescriptor):
+    def MakeAbsoluteSD(pSelfRelativeSecurityDescriptor: Any) -> Any:
         # return a security descriptor in absolute format
         # by using a security descriptor in self-relative format as a template
         pAbsoluteSecurityDescriptor = None
@@ -872,7 +949,7 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
         )
         return pAbsoluteSecurityDescriptor
 
-    def MakeSelfRelativeSD(pAbsoluteSecurityDescriptor):
+    def MakeSelfRelativeSD(pAbsoluteSecurityDescriptor: Any) -> Any:
         # return a security descriptor in self-relative format
         # by using a security descriptor in absolute format as a template
         pSelfRelativeSecurityDescriptor = None
@@ -894,7 +971,7 @@ def _win32_restrict_file_to_user_ctypes(fname: str) -> None:  # noqa
         )
         return pSelfRelativeSecurityDescriptor
 
-    def NewAcl():
+    def NewAcl() -> Any:
         # return a new, initialized ACL (access control list) structure
         nAclLength = 32767  # TODO: calculate this: ctypes.sizeof(ACL) + ?
         acl_data = ctypes.create_string_buffer(nAclLength)
@@ -937,7 +1014,7 @@ def get_file_mode(fname: str) -> int:
     # the missing least significant bit on the third octal digit. In addition, we also tolerate
     # the sticky bit being set, so the lsb from the fourth octal digit is also removed.
     return (
-        stat.S_IMODE(os.stat(fname).st_mode) & 0o6677
+        stat.S_IMODE(Path(fname).stat().st_mode) & 0o6677
     )  # Use 4 octal digits since S_IMODE does the same
 
 
@@ -963,7 +1040,7 @@ def secure_write(fname: str, binary: bool = False) -> Iterator[Any]:
     encoding = None if binary else "utf-8"
     open_flag = os.O_CREAT | os.O_WRONLY | os.O_TRUNC
     try:
-        os.remove(fname)
+        Path(fname).unlink()
     except OSError:
         # Skip any issues with the file not existing
         pass
@@ -976,7 +1053,7 @@ def secure_write(fname: str, binary: bool = False) -> Iterator[Any]:
         else:
             # Python on windows does not respect the group and public bits for chmod, so we need
             # to take additional steps to secure the contents.
-            # Touch file pre-emptively to avoid editing permissions in open files in Windows
+            # Touch file preemptively to avoid editing permissions in open files in Windows
             fd = os.open(fname, open_flag, 0o0600)
             os.close(fd)
             open_flag = os.O_WRONLY | os.O_TRUNC
@@ -986,15 +1063,13 @@ def secure_write(fname: str, binary: bool = False) -> Iterator[Any]:
         if os.name != "nt":
             # Enforce that the file got the requested permissions before writing
             file_mode = get_file_mode(fname)
-            if file_mode != 0o0600:  # noqa
+            if file_mode != 0o0600:
                 if allow_insecure_writes:
                     issue_insecure_write_warning()
                 else:
                     msg = (
-                        "Permissions assignment failed for secure file: '{file}'."
-                        " Got '{permissions}' instead of '0o0600'.".format(
-                            file=fname, permissions=oct(file_mode)
-                        )
+                        f"Permissions assignment failed for secure file: '{fname}'."
+                        f" Got '{oct(file_mode)}' instead of '0o0600'."
                     )
                     raise RuntimeError(msg)
         yield f
@@ -1003,12 +1078,13 @@ def secure_write(fname: str, binary: bool = False) -> Iterator[Any]:
 def issue_insecure_write_warning() -> None:
     """Issue an insecure write warning."""
 
-    def format_warning(msg, *args, **kwargs):
+    def format_warning(msg: str, *args: Any, **kwargs: Any) -> str:  # noqa: ARG001
         return str(msg) + "\n"
 
     warnings.formatwarning = format_warning  # type:ignore[assignment]
     warnings.warn(
         "WARNING: Insecure writes have been enabled via environment variable "
         "'JUPYTER_ALLOW_INSECURE_WRITES'! If this is not intended, remove the "
-        "variable or set its value to 'False'."
+        "variable or set its value to 'False'.",
+        stacklevel=2,
     )
